<?php

    namespace TamerLib\Classes;

    use TamerLib\Exceptions\NoAvailablePortException;

    class Utilities
    {
        /**
         * Attempts to find an available port in the given range. If no port is available,
         * a NoAvailablePortException is thrown.
         *
         * @param string $host
         * @param int $start
         * @param int $end
         * @return int
         * @throws NoAvailablePortException
         */
        public static function getAvailablePort(string $host='127.0.0.1', int $start=1024, int $end=65535): int
        {
            $range = range($start, $end);
            shuffle($range);
            foreach ($range as $port)
            {
                $connection = @stream_socket_client('tcp://' . $host . ':' . $port);
                if (is_resource($connection))
                {
                    fclose($connection);
                }
                else
                {
                    return $port;
                }
            }

            throw new NoAvailablePortException('No available port found in range ' . $start . ' to ' . $end . '.');
        }
    }